import random
import aiosqlite
import os

from .utils import EmbedFactory
from datetime import datetime
from dotenv import load_dotenv
from disnake.ext.commands import Cog, Context, Bot, CheckFailure, command


def get_database() -> str:
    try:
        load_dotenv()
        database_path = os.environ["DATABASE_PATH"]
    except KeyError:
        try:
            os.mkdir("./db")
        except FileExistsError:
            pass
        database_path = "./db/database.db"
    return database_path


class MissingDatabase(Exception):
    """
    aiosqlite missingdatabase exception
    """


class Debug(Cog, name="debug"):
    def __init__(self, bot: Bot) -> None:
        self.bot = bot

        self.setup()
        self.bot.loop.create_task(self.connect_database())

    def get_uptime(self, _type: str = "strict") -> str:
        delta_uptime = datetime.utcnow() - self.bot._launch_time
        hours, remainder = divmod(int(delta_uptime.total_seconds()), 3600)
        minutes, seconds = divmod(remainder, 60)
        days, hours = divmod(hours, 24)

        if _type == "strict":
            return f"{days} days; {hours} hours; {minutes} minutes"
        elif _type == "lazy":
            return f"I've been up for **{days}** Days, **{hours}** Hours, **{minutes}** Minutes, and **{seconds}** Seconds!"

    def setup(self) -> None:
        bot = self.bot

        if not hasattr(bot, "_bot_messages"):
            self.bot._bot_messages = {}
        if not hasattr(bot, "_commands_ran"):
            self.bot._commands_ran = []
        if not hasattr(bot, "_paused"):
            self.bot._paused = False
        if not hasattr(bot, "_launch_time"):
            self.bot._launch_time = datetime.utcnow()
        if not hasattr(bot, "_get_uptime"):
            self.bot._get_uptime = self.get_uptime
        if not hasattr(bot, "_close"):
            self.bot._close = self._close

    async def connect_database(self):
        database_path = get_database()
        try:
            self.bot._db = await aiosqlite.connect(database_path)
        except aiosqlite.OperationalError:
            raise MissingDatabase(f"{database_path} isn't a valid path!")
        except Exception as e:
            print(e)

        cursor = await self.bot._db.cursor()
        await cursor.execute(
            """
            CREATE TABLE IF NOT EXISTS "blacklist" (
                "guild_id"	INTEGER,
                "channel_id" INTEGER,
                "user_id"  INTEGER,
                "commands"	TEXT
            );
            """
        )
        await self.bot._db.commit()

    async def _close(self) -> None:
        await self.bot._db.close()
        await self.bot.close()

    async def bot_check(self, ctx: Context) -> bool:
        if ctx.command.hidden:
            if await self.bot.is_owner(ctx.author):
                return True

        for key, value in {
            "user_id": [ctx.author.id, "you"],
            "channel_id": [ctx.channel.id, "your channel"],
            "guild_id": [ctx.guild.id, "your guild"],
        }.items():
            cursor = await self.bot._db.cursor()
            await cursor.execute(
                f"Select commands from blacklist where {key}=?", (value[0],)
            )
            result = await cursor.fetchone()
            if result:
                result = result[0].split(",")
                if "all" in result:
                    await ctx.send(embed=EmbedFactory.check_fail(ctx, value[1]))
                    return False
                if ctx.command.name in result:
                    await ctx.send(embed=EmbedFactory.check_fail(ctx, value[1]))
                    return False
        return not self.bot._paused

    @command()
    async def t(self, ctx):
        print(".")

    @command()
    async def e(self, ctx):
        print(".")

    @command(hidden=True)
    async def debug(self, ctx):
        """
        echo:
            -> user/channel
                => send message

        blacklist:
            -> user/guild
                => blacklist

        leave_guild:
            -> guild
                => leave guild [confirm]

        change:
            -> avatar/name
                => change avatar/name [confirm]

        invokes:
            -> view all
            -> filter

        eval:
            -> code
                => eval code

        stats:
            => view bot stats (ping, uptime, etc)

        info:
            => view debugger info

        close:
            -> pause
                => pause bot
            -> unpause
                => unpause bot
            -> close
                => close bot [confirm]


        """

        from . import DebugView

        embed = EmbedFactory.static_embed(ctx, "Debug Controls")
        self.bot._bot_messages[ctx.message.id] = await ctx.send(
            embed=embed, view=DebugView(ctx)
        )

    @Cog.listener("on_slash_command")
    async def cache_slash_command(self, interaction):
        _id = random.randint(10000000, 100000000)
        self.bot._commands_ran.append(
            {
                "id": _id,
                "user": interaction.author.name,
                "guild": interaction.guild.name,
                "channel": interaction.channel.name,
                "command": interaction.command.name,
                "errored": interaction.command_failed,
                "bot_paused": self.bot._paused,
                "invoked_with": interaction.invoked_with,
                "message_content": interaction.message.content,
                "user-id": interaction.author.id,
                "guild-id": interaction.guild.id,
                "channel-id": interaction.channel.id,
                "timestamp": datetime.now().strftime("%m/%d/%Y, %H:%M:%S"),
                "raw timestamp": datetime.now(),
            }
        )

    @Cog.listener("on_command")
    async def cache_command(self, ctx):
        _id = random.randint(10000000, 100000000)
        self.bot._commands_ran.append(
            {
                "id": _id,
                "user": ctx.author.name,
                "guild": ctx.guild.name,
                "channel": ctx.channel.name,
                "command": ctx.command.name,
                "errored": ctx.command_failed,
                "bot_paused": self.bot._paused,
                "invoked_with": ctx.invoked_with,
                "message_content": ctx.message.content,
                "user-id": ctx.author.id,
                "guild-id": ctx.guild.id,
                "channel-id": ctx.channel.id,
                "timestamp": datetime.now().strftime("%m/%d/%Y, %H:%M:%S"),
                "raw timestamp": datetime.now(),
            }
        )

    @Cog.listener("on_command_error")
    async def check_failure(self, ctx: Context, error: Exception):
        if isinstance(error, CheckFailure):
            return

        print(error)

    @Cog.listener("on_slash_command_error")
    async def slash_check_failure(self, ctx: Context, error: Exception):
        if isinstance(error, CheckFailure):
            return

        print(error)


def setup(bot: Bot) -> None:
    """Load the Debug extension"""

    bot.add_cog(Debug(bot))



from .constants import EMBED_COLOR
from typing import Union, Optional
from disnake import (
    ButtonStyle,
    Embed,
    Message,
    MessageInteraction,
    Guild,
    TextChannel,
    User,
    Emoji,
    HTTPException,
    utils,
)
from disnake.ui import Button
from disnake.ext.commands import Context, Bot

newline = "\n"
def get_bot_message(ctx: Context) -> Message:
    return ctx.bot._bot_messages[ctx.message.id]
def clean_code(content: str):
    if content.startswith("```py"):
        content = content[5:-3]
    content = content.strip("`")
    content = (
        content.replace("‘", "'").replace("“", '"').replace("”", '"').replace("’", "'")
    )
    return content


class FindSnowflake:
    @staticmethod
    async def find_any(bot: Bot, query: str) -> Union[User, Guild, TextChannel, None]:
        for method in [
            FindSnowflake.find_user,
            FindSnowflake.find_guild,
            FindSnowflake.find_channel,
            FindSnowflake.find_emoji,
        ]:
            response = await method(bot, query)
            if response:
                return response

    @staticmethod
    async def find_user(bot: Bot, query: str) -> Optional[User]:
        if not query.isalpha():  # id
            _id = int(query)

            user = bot.get_user(_id)
            if user:
                return user
            try:
                user = await bot.fetch_user(_id)
                return
            except HTTPException:
                pass

        user = utils.get(bot.users, name=query)
        if user:
            return user

    @staticmethod
    async def find_guild(bot: Bot, query: str) -> Optional[Guild]:
        if not query.isalpha():  # id
            _id = int(query)

            guild = bot.get_guild(_id)
            if guild:
                return guild
            try:
                guild = await bot.fetch_guild(_id)
                return guild
            except HTTPException:
                pass

        guild = utils.get(bot.guilds, name=query)
        if guild:
            return guild

    @staticmethod
    async def find_channel(bot: Bot, query: str) -> Optional[TextChannel]:
        if not query.isalpha():  # id
            _id = int(query)

            channel = bot.get_channel(_id)
            if isinstance(channel, TextChannel):
                return channel
            try:
                channel = await bot.fetch_channel(_id)
                return channel
            except HTTPException:
                pass

        channel = utils.get(bot.get_all_channels(), name=query)
        if channel:
            return channel

    @staticmethod
    async def find_emoji(bot: Bot, query: str) -> Optional[Emoji]:
        if not query.isalpha():  # id
            try:
                _id = int(query)
            except ValueError:
                try:
                    _id = int(query.split(":")[-1][:-1])
                    emoji = bot.get_emoji(_id)
                    if isinstance(emoji, Emoji):
                        return emoji
                except ValueError:
                    pass

        emoji = utils.get(bot.emojis, name=query)
        if emoji:
            return emoji


class EmbedFactory:
    """Embed factory"""

    @staticmethod
    def static_embed(
        ctx: Context,
        title: str,
        *,
        path: str = None,
        description: str = None,
        markdown: str = "yaml",
    ) -> Embed:
        return (
            Embed(
                title=title,
                colour=EMBED_COLOR,
                description=f"```yaml\nCurrent path: /{path or ''}```{'```' + markdown + newline + description + '```' if description else ''}",
                timestamp=ctx.message.created_at,
            )
            .set_author(name=ctx.author.name, icon_url=ctx.author.avatar.url)
            .set_footer(
                text=f"Bot ping: {ctx.bot.latency * 1000:.0f} || Uptime: {ctx.bot._get_uptime()}"
            )
        )

    @staticmethod
    def check_fail(
        ctx: Context,
        description: str,
    ) -> Embed:
        return Embed(
            title="Uh oh",
            description=f"One of my owners has `blacklisted` {description} from using this command.",
            colour=EMBED_COLOR,
        ).set_author(name=ctx.author.name, icon_url=ctx.author.avatar.url)

    @staticmethod
    def error_embed(
        ctx: Context,
        reason: str,
    ) -> Embed:
        return Embed(
            title="Uh oh",
            description=f"```yaml\n{reason}```",
            timestamp=ctx.message.created_at,
            colour=EMBED_COLOR,
        ).set_author(name=ctx.author, icon_url=ctx.author.avatar.url)


class MainMenu(Button):
    def __init__(self, ctx: Context):
        super().__init__(style=ButtonStyle.danger, label="Back")
        self.ctx = ctx
        self.bot_message = get_bot_message(ctx)

    async def callback(self, interaction: MessageInteraction):
        from . import DebugView

        embed = EmbedFactory.static_embed(self.ctx, "Debug Controls")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=DebugView(self.ctx))



import random

from ..utils import FindSnowflake, EmbedFactory, MainMenu, get_bot_message
from ..constants import THUMBS_UP, ERROR
from typing import Union

from disnake import Message, MessageInteraction, ButtonStyle, TextChannel, Guild
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot

user_info = """
Name: {0.name}
Discriminator: {0.discriminator}
Id: {0.id}
Joined discord: {joined_at}
Flags: {flags}
"""

channel_info = """
Name: {0.name}
Id: {0.id}
Created at: {created_at}
Invite: {invite}
"""

guild_info = """
Name: {0.name}
Id: {0.id}
Owner: {0.owner.name}#{0.owner.descriminator}
Members: {members}
Bots: {bots}
Roles: {roles}
Channels: {channels}
Created at: {created_at}
Invite: {invite}
"""

emoji_info = """
Name: {0.name}
Id: {0.id}
Guild id: {0.guild.id}
Created at: {created_at}
"""


class ViewInfo(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    async def get_invite(self, channel: Union[TextChannel, Guild]) -> str:
        if isinstance(channel, Guild):
            channel = random.choice(channel.text_channels)
        return await channel.create_invite()

    @button(label="User", style=ButtonStyle.green)
    async def user_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/user",
            description=f"Send an id/name of the user you want to view",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_user(self.bot, message.content)
        if not response:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "View",
                path=f"view/user/notfound",
                description=f"I could not find a user with id/name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/user/info",
            description=user_info.format(
                response,
                joined_at=response.created_at.strftime("%m/%d/%Y"),
                flags=", ".join([f"{k.name}" for k in response.public_flags.all()]),
            ),
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)

    @button(label="Channel", style=ButtonStyle.green)
    async def channel_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/channel",
            description=f"Send an id/name of the channel you want to view",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_channel(self.bot, message.content)
        if not response:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "View",
                path=f"view/channel/notfound",
                description=f"I could not find a channel with id/name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/channel/info",
            description=channel_info.format(
                response,
                created_at=response.created_at.strftime("%m/%d/%Y"),
                invite=await self.get_invite(response),
            ),
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)

    @button(label="Guild", style=ButtonStyle.green)
    async def guild_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/guild",
            description=f"Send an id/name of the guild you want to view",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_guild(self.bot, message.content)
        if not response:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "View",
                path=f"view/guild/notfound",
                description=f"I could not find a guild with id/name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/guild/info",
            description=guild_info.format(
                response,
                created_at=response.created_at.strftime("%m/%d/%Y"),
                invite=await self.get_invite(response),
                bots=len([k for k in response.members if k.bot]),
                channels=len(response.text_channels),
                roles=len(response.roles),
                members=len([k for k in response.members if not k.bot]),
            ),
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)

    @button(label="Emoji", style=ButtonStyle.green)
    async def emoji_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "View",
            path=f"view/emoji",
            description=f"Send an id/name of the emoji you want to view",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_emoji(self.bot, message.content)
        if not response:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "View",
                path=f"view/emoji/notfound",
                description=f"I could not find an emoji with id/name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)
        embed = EmbedFactory.static_embed(
            self.ctx,
            f"View Emoji: {response}",
            path=f"view/emoji/info",
            description=emoji_info.format(
                response,
                created_at=response.created_at.strftime("%m/%d/%Y"),
            ),
        )
        embed.description += f"**Emoji:** {response}"
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)



import disnake
import io
import contextlib
import textwrap

from traceback import format_exception
from disnake_paginator import ButtonPaginator
from ..utils import EmbedFactory, MainMenu, clean_code, get_bot_message
from ..constants import EMBED_COLOR

from disnake import MessageInteraction, ButtonStyle, Message
from disnake.ui import View, Button, button
from disnake.ext import commands
from disnake.ext.commands import Context, Bot


async def eval_code(
    message: Message, local_variables: dict, method: str = "None"
) -> str:
    code = clean_code(message.content)
    message = clean_code(message.content)

    stdout = io.StringIO()
    if method == "dir":
        code = f"print(dir({code}))"

    elif method == "return":
        code = f"return {code}"

    try:
        with contextlib.redirect_stdout(stdout):
            exec(
                f"async def func():\n{textwrap.indent(code, '    ')}",
                local_variables,
            )
            obj = await local_variables["func"]()
            result = f"{stdout.getvalue()}{obj}\n"

    except Exception as e:
        result = "".join(format_exception(e, e, e.__traceback__))
        pass

    result = result.replace("`", "")
    message = message.replace("`", "")

    if result.replace("\n", "").endswith("None") and result != "None":
        result = result[:-5]
    if len(result) < 2000:
        return f"In: {message}\nOut: {result}"

    return [result[i : i + 2000] for i in range(0, len(result), 2000)]


class Eval(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.vars = {
            "disnake": disnake,
            "commands": commands,
            "bot": ctx.bot,
            "client": ctx.bot,
            "ctx": ctx,
            "channel": ctx.channel,
            "author": ctx.author,
            "guild": ctx.guild,
            "message": ctx.message,
        }

        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="Eval", style=ButtonStyle.green)
    async def eval_button(self, button: Button, interaction: MessageInteraction):
        """
        evals content
        """

        embed = EmbedFactory.static_embed(self.ctx, "Evaluate code", path="eval/code")

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What code would you like to evaluate?", ephemeral=True
        )

        message: Message = await self.bot.wait_for(
            "message",
            check=lambda m: m.author == self.ctx.author
            and m.channel == self.ctx.channel,
        )
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = await eval_code(message, self.vars)

        if not isinstance(output, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Evaluate code",
                path="eval/code/run",
                description=output,
                markdown="py",
            )
            return await self.bot_message.edit(embed=embed)
        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="Return Eval", style=ButtonStyle.green)
    async def return_eval_button(self, button: Button, interaction: MessageInteraction):
        """
        returns content
        """

        embed = EmbedFactory.static_embed(self.ctx, "Evaluate code", path="eval/code")

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What code would you like to evaluate?", ephemeral=True
        )

        message: Message = await self.bot.wait_for(
            "message",
            check=lambda m: m.author == self.ctx.author
            and m.channel == self.ctx.channel,
        )
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = await eval_code(message, self.vars, "return")

        if not isinstance(output, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Evaluate code",
                path="eval/code/run",
                description=output,
                markdown="py",
            )
            return await self.bot_message.edit(embed=embed)
        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="Dir Eval", style=ButtonStyle.green)
    async def dir_eval_button(self, button: Button, interaction: MessageInteraction):
        """
        prints the dir of content
        """

        embed = EmbedFactory.static_embed(self.ctx, "Evaluate code", path="eval/code")

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What code would you like to evaluate?", ephemeral=True
        )

        message: Message = await self.bot.wait_for(
            "message",
            check=lambda m: m.author == self.ctx.author
            and m.channel == self.ctx.channel,
        )
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = await eval_code(message, self.vars, "dir")

        if not isinstance(output, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Evaluate code",
                path="eval/code/run",
                description=output,
                markdown="py",
            )
            return await self.bot_message.edit(embed=embed)
        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)



from ..utils import EmbedFactory, MainMenu, get_bot_message
from fuzzywuzzy import fuzz
from typing import List

from disnake import MessageInteraction, Message, SelectOption, Guild, Forbidden, ButtonStyle
from disnake.ui import Select, View, Button, button
from disnake.ext.commands import Bot, Context


class Utilities:
    def __init__(self, ctx: Context) -> None:
        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)

    async def other_guilds(self, interaction: MessageInteraction, clicked: str) -> None:
        guilds: List[Guild] = [
            k for k in self.bot.guilds if ord(k.name.lower()) not in range(96, 123)
        ]
        if not guilds:
            embed = EmbedFactory.static_embed(
                self.ctx,
                f"Leave guild",
                path="leave_guild/notfound",
                description="I could not find any guilds that have special characters at the start!",
            )
            await interaction.response.defer()
            return await self.bot_message.edit(embed=embed)

        embed = EmbedFactory.static_embed(
            self.ctx,
            f"Leave guild",
            path="leave_guild/choose",
            description="I am in these servers:\n"
            + "\n".join([f"-> guild.name" for guild in guilds]),
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "Which guild would you like me to leave?"
        )
        for item in self.bot_message.components:
            if isinstance(item, Select):
                item.disabled = True
        await self.bot_message.edit(embed=embed, view=self)

        message: Message = await self.bot.wait_for(
            "message",
            check=lambda m: m.author == self.ctx.author
            and m.channel == self.ctx.channel,
        )
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        selected_guild = max(
            [[guild, fuzz.ratio(message.content, guild.name)] for guild in guilds],
            key=lambda m: m[1],
        )

        embed = EmbedFactory.static_embed(
            self.ctx,
            f"Leave guild",
            path="leave_guild/choose/confirm",
            description=f"Are you sure you want me to leave {selected_guild.name}?",
        )
        await self.bot_message.edit(
            embed=embed, view=LeaveGuildConfirm(self.ctx, selected_guild)
        )

    async def find_guilds(self, interaction: MessageInteraction, clicked: str) -> None:
        guilds: List[Guild] = [
            k for k in self.bot.guilds if k.name.lower().startswith(clicked)
        ]
        if not guilds:
            embed = EmbedFactory.static_embed(
                self.ctx,
                f"Leave guild",
                path="leave_guild/notfound",
                description=f"I could not find any guilds starting with {clicked}",
            )
            await interaction.response.defer()
            return await self.bot_message.edit(embed=embed)

        embed = EmbedFactory.static_embed(
            self.ctx,
            f"Leave guild",
            path="leave_guild/choose",
            description=", ".join([guild.name for guild in guilds]),
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "Which guild would you like me to leave?"
        )
        view = View()
        view.add_item(MainMenu(self.ctx))
        await self.bot_message.edit(embed=embed, view=view)

        message: Message = await self.bot.wait_for(
            "message",
            check=lambda m: m.author == self.ctx.author
            and m.channel == self.ctx.channel,
        )
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        selected_guild = max(
            [[guild, fuzz.ratio(message.content, guild.name)] for guild in guilds],
            key=lambda m: m[1],
        )[0]

        embed = EmbedFactory.static_embed(
            self.ctx,
            f"Leave guild",
            path="leave_guild/choose/confirm",
            description=f"Are you sure you want me to leave {selected_guild.name}?",
        )
        await self.bot_message.edit(
            embed=embed, view=LeaveGuildConfirm(self.ctx, selected_guild)
        )


class LeaveGuildConfirm(View):
    def __init__(self, ctx: Context, guild: Guild):
        super().__init__()

        self.ctx = ctx
        self.bot_message = get_bot_message(ctx)
        self.guild = guild
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="Yes", style=ButtonStyle.green)
    async def confirm_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            f"Leave guild",
            path="leave_guild/leave",
            description=f"Left {self.guild.name}",
        )

        await self.bot_message.edit(embed=embed)
        await self.guild.leave()

        try:
            await self.bot_message.edit(view=LeaveGuild(self.ctx))
        except Forbidden:
            pass

    @button(label="No", style=ButtonStyle.danger)
    async def cancel_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            f"Leave guild",
            path="leave_guild/cancel",
            description=f"Didn't leave {self.guild.name}",
        )
        await self.bot_message.edit(embed=embed, view=LeaveGuild(self.ctx))


class SelectPZ(Select):
    def __init__(self, ctx: Context):
        super().__init__()

        self.bot: Bot = ctx.bot
        self.utils = Utilities(ctx)
        self.options = [
            SelectOption(label=str(k), value=str(k))
            for k in [chr(a) for a in range(112, 123)]
        ]
        self.options.append(SelectOption(label="Other", value="other"))

    async def callback(self, interaction: MessageInteraction) -> None:
        clicked = self.values[0]
        if clicked == "other":
            return await self.utils.find_other_guilds(interaction)
        await self.utils.find_guilds(interaction)


class SelectAO(Select):
    def __init__(self, ctx: Context):
        super().__init__()

        self.bot: Bot = ctx.bot
        self.utils = Utilities(ctx)
        self.options = [
            SelectOption(label=str(k), value=str(k))
            for k in [chr(a) for a in range(97, 112)]
        ]
        self.options.append(SelectOption(label="Other", value="other"))

    async def callback(self, interaction: MessageInteraction) -> None:
        clicked = self.values[0]
        if clicked == "other":
            return await self.utils.find_other_guilds(interaction, clicked)
        await self.utils.find_guilds(interaction, clicked)


class LeaveGuild(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.add_item(SelectAO(ctx))
        self.add_item(SelectPZ(ctx))
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )



from .utils import FindSnowflake, EmbedFactory, get_bot_message
from .constants import DESCRIPTION

from disnake import ButtonStyle, Message, MessageInteraction, TextChannel
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot

newline = "\n"
bot_statistics = """
-> Bot ping: {0}    
-> Bot name: {1.user.name}
-> Bot owners: {1.owner_ids}
-> Bot uptime: {2}
-> Total invokes: {3}
-> Servers in: {4}
-> Users: {5}
-> Channels: {6}
-> Prefix: {7}
-> Number of extensions loaded: {8}
"""

class DebugView(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        self.bot_message = get_bot_message(self.ctx)

        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    @button(label="Blacklist", style=ButtonStyle.green)
    async def blacklist_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> user
            -> channel
                => blacklist user/channel
        """

        from . import Blacklist

        embed = EmbedFactory.static_embed(
            self.ctx, "Blacklist/Unblacklist", path="blacklist"
        )

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Blacklist(self.ctx))

    @button(label="Change", style=ButtonStyle.green)
    async def change_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> avatar
                => change avatar
            -> name
                => change name
        """

        from . import Change

        embed = EmbedFactory.static_embed(
            self.ctx, "Change bot information", path="change"
        )

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Change(self.ctx))

    @button(label="Cogs", style=ButtonStyle.green)
    async def cogs_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> load
                => load cog
            -> unload
                => unload cog
            -> reload
                => reload cog/all
        """

        from . import Cogs

        embed = EmbedFactory.static_embed(self.ctx, "Manage cogs", path="cogs")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Cogs(self.ctx))

    @button(label="Echo", style=ButtonStyle.green)
    async def echo_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> send message
            -> delete message
        """

        from . import Channel

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Speak as the bot",
            path=f"echo/channel",
            description=f"Send an id/name of the channel you want to view and send messages in",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_channel(self.bot, message.content)
        if not response:
            other_response = await FindSnowflake.find_any(self.bot, message.content)
            if not other_response:
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Speak as the bot",
                    path=f"echo/channel/notfound",
                    description=f"I could not find channel with id/name {message.content}",
                )
                return await self.bot_message.edit(embed=embed)
            if not isinstance(other_response, TextChannel):
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Speak as the bot",
                    path=f"echo/channel/other",
                    description=f"I could not find a channel with id/name {message.content}, but i found a {type(other_response)} {other_response}",
                )
                return await self.bot_message.edit(embed=embed)

        if isinstance(response, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Speak as the bot",
                path=f"echo/channel/overload",
                description=f"You were too broad! I found multiple **channels** with name {message.content}",
            )
            return await self.bot_message.edit(embed=embed)

        await interaction.send(
            "Loading messages...this might take a while if the channel contains long messages"
        )
        view = await Channel.create(self.ctx, response)
        await self.bot_message.edit(view=view)

    @button(label="Eval", style=ButtonStyle.green)
    async def eval_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> eval
            -> return eval
            -> dir eval

                => output
        """

        from . import Eval

        embed = EmbedFactory.static_embed(self.ctx, "Evaluate code", path="eval")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Eval(self.ctx))

    @button(label="Github", style=ButtonStyle.green, row=1)
    async def github_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> pull
            -> push
        """

        from . import Github

        embed = EmbedFactory.static_embed(self.ctx, "Github", path="github")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Github(self.ctx))

    @button(label="Invokes", style=ButtonStyle.green, row=1)
    async def invoke_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> all
                => all invokes
            -> find
                -> ID
                -> USER
                -> CHANNEL
                -> GUILD
                    => search and return results
        """

        from . import Invokes

        embed = EmbedFactory.static_embed(self.ctx, "Invokes", path="invokes")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Invokes(self.ctx))

    @button(label="Leave guild", style=ButtonStyle.green, row=1)
    async def leave_guild_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> find
                => leave guild [confirm]
        """

        from . import LeaveGuild

        embed = EmbedFactory.static_embed(self.ctx, "Leave guild", path="leave_guild")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=LeaveGuild(self.ctx))

    @button(label="Generate invite", style=ButtonStyle.blurple, row=1)
    async def generate_invite_button(
        self, button: Button, interaction: MessageInteraction
    ):
        """
        buttons:
            none
        """

        from .dialogs import Invite

        embed = EmbedFactory.static_embed(self.ctx, "Generate invite", path="invite")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Invite(self.ctx))

    @button(label="Go to", style=ButtonStyle.blurple, row=2)
    async def go_to_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            none
        """

        from .dialogs import Blacklist, Change, Eval, Guild, Invokes, Manage, ViewInfo

        paths = {
            "blacklist": Blacklist,
            "change": Change,
            "eval": Eval,
            "invokes": Invokes,
            "leave_guild": Guild,
            "manage": Manage,
            "view": ViewInfo,
        }
        visual_paths = newline.join(["  -> /" + k for k in list(paths.keys())])
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Go to path",
            path="go_to",
            description=f"Type the path you want to go to eg /blacklist/user\nNot all paths are available because some require additional info\nPaths are:\n{visual_paths}",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What path?", ephemeral=True)

        message: Message
        while (
            (message := await self.bot.wait_for("message", check=self.check))
        ).content.lower() != "q":
            content = message.content.lower()

            if content in paths:
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    content.capitalize(),
                    path=f"{content}",
                    description=f"Moved to path {content}",
                )
                return await self.bot_message.edit(
                    embed=embed, view=paths[content](self.ctx)
                )

            embed = EmbedFactory.static_embed(
                self.ctx,
                "Go to path",
                path="go_to",
                description=f"That is not a valid path\nPaths are:\n{visual_paths}",
            )
            await self.bot_message.edit(embed=embed)
            await interaction.message.reply("Invalid path, try again")
        return await interaction.message.reply("Cancelled the current `wait_for`")

    @button(label="View", style=ButtonStyle.blurple, row=2)
    async def view_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            -> user
            -> channel
            -> guild
            -> emoji
            -> any

                => show info
        """

        from . import ViewInfo

        embed = EmbedFactory.static_embed(self.ctx, "View", path="view")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=ViewInfo(self.ctx))

    @button(label="Stats", style=ButtonStyle.blurple, row=2)
    async def stats_button(self, button: Button, interaction: MessageInteraction):

        """
        buttons:
            none
        """

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Stats",
            path="stats",
            description=bot_statistics.format(
                round(self.bot.latency * 1000),
                self.bot,
                self.bot._get_uptime(),
                len(self.bot._commands_ran),
                len(self.bot.guilds),
                len(self.bot.users),
                len([e for e in self.bot.get_all_channels()]),
                self.bot.command_prefix,
                len(self.bot.extensions),
            ),
        )

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed)

    @button(label="Help", style=ButtonStyle.blurple, row=2)
    async def help_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            none
        """

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Information about disnake-debug",
            path="help",
            description=DESCRIPTION,
        )

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed)

    @button(label="Manage bot", style=ButtonStyle.danger, row=2)
    async def manage_button(self, button: Button, interaction: MessageInteraction):
        """
        buttons:
            => close [confirm]
        """

        from . import Manage

        embed = EmbedFactory.static_embed(self.ctx, "Manage bot", path="manage")

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=Manage(self.ctx))



from ..utils import EmbedFactory, MainMenu, get_bot_message

from disnake import HTTPException, Message, MessageInteraction, ButtonStyle
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot


class Change(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    @button(label="Name", style=ButtonStyle.green)
    async def name_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx, "Change", path="change/name", description="Changing my name"
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What username?", ephemeral=True)

        message: Message
        while len(
            ((message := await self.bot.wait_for("message", check=self.check))).content
        ) not in range(2, 33):
            await interaction.message.reply(
                "My username must be between `2-32` characters in length"
            )
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        try:
            await self.bot.user.edit(username=message.content)
        except HTTPException:
            await interaction.message.reply(
                "Something went wrong - Maybe you have been changing my name too often"
            )

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Change",
            path="change/name/success",
            description=f"I changed my name to {message.content}",
        )
        await self.bot_message.edit(embed=embed)

    @button(label="Avatar", style=ButtonStyle.green)
    async def avatar_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Change",
            path="change/avatar",
            description="Changing my avatar - Send an image",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What avatar? send an image", ephemeral=True
        )

        message: Message
        while not (
            (message := await self.bot.wait_for("message", check=self.check))
        ).attachments:
            await interaction.message.reply("You must send an image file!")

        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        try:
            await self.bot.user.edit(avatar=await message.attachments[0].read())
        except HTTPException:
            await interaction.message.reply(
                "Something went wrong - Maybe you have been changing my avatar too often"
            )

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Change",
            path="change/avatar/success",
            description=f"I changed my avatar successfully!",
        )
        await self.bot_message.edit(embed=embed)



from ..utils import EmbedFactory, MainMenu, get_bot_message
from ..constants import PERMISSION_LIST

from disnake import MessageInteraction, ButtonStyle, Message
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot


class Invite(View):
    def __init__(self, ctx: Context, *, restart: bool = False):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.restart = restart
        self.bot_message = get_bot_message(ctx)
        self.base_url = "https://discord.com/oauth2/authorize?client_id={bot_id}&scope=bot{application}&permissions={permissions}"
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    @button(label="No slash commands", style=ButtonStyle.green)
    async def bot_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Generate invite",
            path="invite/bot",
            description=f"What permissions?{PERMISSION_LIST}",
        )
        await self.bot_message.edit(embed=embed, view=Invite(self.ctx))
        await interaction.response.send_message("What permission value?")

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        invite = self.base_url.format(
            bot_id=self.bot.user.id, application="", permissions=message.content
        )
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Generate invite",
            path="invite/bot/create",
            description="Generated invite: " + invite,
        )
        await interaction.message.reply(f"Invite: {invite}")
        await self.bot_message.edit(embed=embed)

    @button(label="Slash commands", style=ButtonStyle.green)
    async def slash_command_button(
        self, button: Button, interaction: MessageInteraction
    ):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Generate invite",
            path="invite/application",
            description=f"What permissions?{PERMISSION_LIST}",
        )

        await self.bot_message.edit(embed=embed, view=Invite(self.ctx))
        await interaction.response.send_message("What permission value?")

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        invite = self.base_url.format(
            bot_id=self.bot.user.id,
            application="%20applications.commands",
            permissions=message.content,
        )
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Generate invite",
            path="invite/application/create",
            description="Generated invite: " + invite,
        )
        await interaction.message.reply(f"Invite: {invite}")
        await self.bot_message.edit(embed=embed)



from .blacklist import *
from .change import *
from .cogs import *
from .echo import *
from .eval import *
from .github import *
from .leave_guild import *
from .invite import *
from .invoke import *
from .manage import *
from .view import *



import asyncio
import os
import __main__
from ..utils import EmbedFactory, MainMenu, get_bot_message

from disnake import MessageInteraction, ButtonStyle
from disnake.ui import View, Button, button
from disnake.ext.commands import Context

newline = "\n"


class ConfirmShutdown(View):
    def __init__(self, ctx: Context, *, restart: bool = False):
        super().__init__()

        self.ctx = ctx
        self.restart = restart
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    async def restart_bot(self) -> None:
        """
        restart bot, if stuck in event loop stderr will never be created, hence triggering bot shut down
        """

        path = os.getcwd()
        main = str(__main__.__file__).split("/")[-1]

        proc = await asyncio.create_subprocess_shell(
            f"cd {path} && python {main}",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        self.stdout, self.stderr = await proc.communicate()
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Uh oh!",
            path=f"manage/restart/error",
            description=f"stderr: {self.stderr.decode()}\nstdout: {self.stdout.decode() if self.stdout else 'no stdout'}{newline + 'I am not shutting of the bot because I would error on startup' if self.stderr else ''}",
        )
        await self.bot_message.edit(embed=embed)

    @button(label="Yes", style=ButtonStyle.green)
    async def confirm_button(self, button: Button, interaction: MessageInteraction):
        """
        close/restart bot
        """

        await interaction.response.defer()
        if not self.restart:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Turned off bot",
                path=f"manage/shut_down",
                description=f"The bot is now offline",
            )
            await self.bot_message.edit(embed=embed)
            return await self.ctx.bot.close()

        asyncio.create_task(self.restart_bot())
        await asyncio.sleep(3)
        try:
            self.stderr
        except AttributeError:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Turned off bot",
                path=f"manage/restart",
                description=f"The bot is now offline",
            )
            await self.bot_message.edit(embed=embed)
            return await self.ctx.bot.close()

    @button(label="No", style=ButtonStyle.danger)
    async def cancel_button(self, button: Button, interaction: MessageInteraction):
        from . import DebugView

        embed = EmbedFactory.static_embed(self.ctx, "Debug Controls")
        await self.bot_message.edit(embed=embed, view=DebugView(self.ctx))


class Manage(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="Pause", style=ButtonStyle.green)
    async def pause_button(self, button: Button, interaction: MessageInteraction):
        """
        pause the bot so it does not respond to commands
        """

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Paused bot",
            path="manage/pause",
            description="Bot paused; commands locked",
        )
        bot = self.ctx.bot
        bot._paused = True

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed)

    @button(label="Resume", style=ButtonStyle.green)
    async def resume_button(self, button: Button, interaction: MessageInteraction):
        """
        unpause the bot
        """

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Resumed bot",
            path="manage/resume",
            description="Resumed bot; commands unlocked",
        )
        bot = self.ctx.bot
        bot._paused = False

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed)

    @button(label="Restart", style=ButtonStyle.danger)
    async def restart_button(self, button: Button, interaction: MessageInteraction):
        """
        restart the bot
        """

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Are you sure you want to restart?",
            path="manage/restart",
            description="WARNING: If there is an error on startup, the bot will not be able to start back up",
        )

        await interaction.response.defer()
        await self.bot_message.edit(
            embed=embed, view=ConfirmShutdown(self.ctx, restart=True)
        )

    @button(label="Shut down", style=ButtonStyle.danger)
    async def shutdown_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Are you sure you want to shut the bot down?",
            path="manage/shut_down",
        )

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=ConfirmShutdown(self.ctx))



from ..utils import EmbedFactory, MainMenu, get_bot_message
from ..constants import ERROR, THUMBS_UP

from disnake import Message, MessageInteraction, ButtonStyle
from disnake.ui import View, Button, button
from disnake.ext.commands import (
    Context,
    Bot,
    ExtensionNotFound,
    ExtensionAlreadyLoaded,
    ExtensionNotLoaded,
)


class Cogs(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))
        cog_not_found = EmbedFactory.error_embed(self.ctx, "Extension not found")
        cog_not_loaded = EmbedFactory.error_embed(self.ctx, "Extension not loaded")
        cog_already_loaded = EmbedFactory.error_embed(self.ctx, "Extension already")

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    @button(label="Load", style=ButtonStyle.green)
    async def load_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Manage cogs",
            path="cog/load",
            description="What is the file path of the cog you want to load?",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What cog do you want to load?", ephemeral=True
        )

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        fp = message.content.replace("/", ".")

        try:
            self.bot.load_extension(fp)
        except ExtensionNotFound:
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=self.cog_not_found)
        except ExtensionAlreadyLoaded:
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=self.cog_already_loaded)

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Manage cogs",
            path="cog/load/success",
            description=f"Successfully loaded {fp}",
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)

    @button(label="Unload", style=ButtonStyle.green)
    async def unload_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Manage cogs",
            path="cog/unload",
            description="What is the file path of the cog you want to unload?",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What cog do you want to unload?", ephemeral=True
        )

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        fp = message.content.replace("/", ".")

        try:
            self.bot.load_extension(fp)
        except ExtensionNotFound:
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=self.cog_not_found)
        except ExtensionNotLoaded:
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=self.cog_not_loaded)

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Manage cogs",
            path="cog/unload/success",
            description=f"Successfully unloaded {fp}",
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)

    @button(label="Reload", style=ButtonStyle.green)
    async def reload_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Manage cogs",
            path="cog/reload",
            description="What is the file path of the cog you want to reload?",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            "What cog do you want to reload?", ephemeral=True
        )

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        fp = message.content.replace("/", ".")

        try:
            self.bot.load_extension(fp)
        except ExtensionNotFound:
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=self.cog_not_found)
        except ExtensionNotLoaded:
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=self.cog_not_loaded)

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Manage cogs",
            path="cog/reload/success",
            description=f"Successfully reloaded {fp}",
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(embed=embed)



from typing import List
from ..utils import MainMenu, get_bot_message
from ..constants import THUMBS_UP, EMBED_COLOR

from disnake import (
    HTTPException,
    TextChannel,
    Message,
    MessageInteraction,
    ButtonStyle,
    Embed,
)
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot


class Channel(View):
    def __init__(self, ctx: Context, channel: TextChannel):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.channel = channel
        self.bot_message = get_bot_message(ctx)
        self.embed = Embed(
            title="Viewing: " + channel.name,
            timestamp=self.ctx.message.created_at,
            colour=EMBED_COLOR,
        )
        self.messages: List[Message] = []
        self.bot.add_listener(self.specific_channel_message, "on_message")
        self.bot.add_listener(self.specific_channel_message_delete, "on_message_delete")
        self.bot.add_listener(self.specific_channel_message_edited, "on_message_edit")
        self.add_item(MainMenu(ctx))

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @classmethod
    async def create(cls, ctx: Context, channel: TextChannel):
        self = Channel(ctx, channel)
        await self.load_messages()
        return self

    async def load_messages(self) -> None:
        self.messages = [
            f"[{message.id}] {message.author.name}: {message.content}"
            for message in await self.channel.history(limit=30).flatten()
        ][::-1]
        self.embed.description = "```yaml\n" + "\n".join(self.messages) + "```"
        i = 29
        while True:
            try:
                await self.bot_message.edit(embed=self.embed)
                return
            except HTTPException:
                i -= 2
                self.messages = [
                    f"[{message.id}] {message.author.name}: {message.content}"
                    for message in await self.channel.history(limit=i).flatten()
                ][::-1]
                self.embed.description = "```yaml\n" + "\n".join(self.messages) + "```"

    async def update_embed(self, message: Message) -> None:
        self.messages.pop(0)
        self.messages.append(f"[{message.id}] {message.author.name}: {message.content}")
        self.embed.description = "```yaml\n" + "\n".join(self.messages) + "```"

        while True:
            try:
                await self.bot_message.edit(embed=self.embed)
                return
            except HTTPException:
                self.messages.pop(0)
                self.embed.description = "```yaml\n" + "\n".join(self.messages) + "```"

    async def message_deleted(self, message: Message) -> None:
        if not any(str(message.id) in msg for msg in self.messages):
            return
        self.messages.remove(f"[{message.id}] {message.author.name}: {message.content}")
        self.embed.description = "```yaml\n" + "\n".join(self.messages) + "```"
        await self.bot_message.edit(embed=self.embed)

    async def message_edited(self, before: Message, after: Message) -> None:
        if not any(str(before.id) in msg for msg in self.messages):
            return
        self.messages[
            self.messages.index(f"[{before.id}] {before.author.name}: {before.content}")
        ] = f"[{after.id}] {after.author.name}: {after.content}"
        self.embed.description = "```yaml\n" + "\n".join(self.messages) + "```"
        await self.bot_message.edit(embed=self.embed)

    async def specific_channel_message(self, message: Message) -> None:
        if message.channel.id != self.channel.id:
            return
        if message.author.bot and message.author != self.bot.user:
            return
        await self.update_embed(message)

    async def specific_channel_message_delete(self, message: Message) -> None:
        if message.channel.id != self.channel.id:
            return
        if message.author.bot and message.author != self.bot.user:
            return
        await self.message_deleted(message)

    async def specific_channel_message_edited(
        self, before: Message, after: Message
    ) -> None:
        if before.channel.id != self.channel.id:
            return
        if before.author.bot and before.author != self.bot.user:
            return
        await self.message_edited(before, after)

    @button(label="Send message", style=ButtonStyle.green)
    async def send_message_button(
        self, button: Button, interaction: MessageInteraction
    ):
        if not self.channel.permissions_for(self.ctx.me).send_messages:
            return await interaction.response.send_message(
                "I do not have permissions to send messages", ephemeral=True
            )

        await interaction.response.send_message(
            "What would you like to say?", ephemeral=True
        )
        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        await self.channel.send(message.content)

    @button(label="Delete message", style=ButtonStyle.green)
    async def delete_message_button(
        self, button: Button, interaction: MessageInteraction
    ):
        if not self.channel.permissions_for(self.ctx.me).manage_messages:
            return await interaction.response.send_message(
                "I do not have permissions to delete messages", ephemeral=True
            )

        await interaction.response.send_message("What message id?", ephemeral=True)
        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        try:
            deleteable = await self.channel.fetch_message(int(message.content))
        except HTTPException:
            return await interaction.send(
                f"Could not find message with id {message.content}"
            )

        await deleteable.delete()
        await message.add_reaction(THUMBS_UP)



from typing import List
from disnake_paginator import ButtonPaginator
from fuzzywuzzy import fuzz
from ..utils import EmbedFactory, MainMenu, get_bot_message
from ..constants import EMBED_COLOR

from disnake import Message, MessageInteraction, ButtonStyle
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot


def pretty(invokes: List[dict]) -> List[str]:
    output = []
    for iteration, invoke_dict in enumerate(invokes):
        output.append(
            "\n".join(
                [
                    f"-> {key}: {invokes[iteration][key]}"
                    for key in list(invoke_dict.keys())
                ]
            )
        )
    return output[::-1]


def find(invokes: List[dict], *, key: str, value: str) -> List[str]:
    output = [
        invoke_dict
        for num, invoke_dict in enumerate(invokes)
        if fuzz.ratio(str(invokes[num][key]), value.lower()) > 70
    ]
    return pretty(output)


class FindInvoke(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="ID", style=ButtonStyle.green)
    async def id_button(self, button: Button, interaction: MessageInteraction):
        name = "id"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"What {name}?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="USER_NAME", style=ButtonStyle.green)
    async def user_button(self, button: Button, interaction: MessageInteraction):
        name = "user"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"What {name}?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="GUILD_NAME", style=ButtonStyle.green)
    async def guild_button(self, button: Button, interaction: MessageInteraction):
        name = "guild"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"What {name}?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="CHANNEL_NAME", style=ButtonStyle.green)
    async def channel_button(self, button: Button, interaction: MessageInteraction):
        name = "channel"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"What {name}?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="COMMAND_NAME", style=ButtonStyle.green)
    async def command_button(self, button: Button, interaction: MessageInteraction):
        name = "command"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"What {name}?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="ERRORED", style=ButtonStyle.green)
    async def errored_button(self, button: Button, interaction: MessageInteraction):
        name = "errored"

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Find Invoke",
            path=f"invokes/find/{name}",
            description="True/False",
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"True/False?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="BOT_PAUSED", style=ButtonStyle.green)
    async def bot_paused_button(self, button: Button, interaction: MessageInteraction):
        name = "bot_paused"

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Find Invoke",
            path=f"invokes/find/{name}",
            description="True/False",
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"True/False?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="INVOKED_WITH", style=ButtonStyle.green)
    async def invoked_with_button(
        self, button: Button, interaction: MessageInteraction
    ):
        name = "invoked_with"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(
            f"What was the command {name}?", ephemeral=True
        )

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="MESSAGE_CONTENT", style=ButtonStyle.green)
    async def message_content_button(
        self, button: Button, interaction: MessageInteraction
    ):
        name = "message_content"

        embed = EmbedFactory.static_embed(
            self.ctx, "Find Invoke", path=f"invokes/find/{name}"
        )

        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message(f"What {name}?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        output = find(self.bot._commands_ran, key=name, value=message.content)

        if not output:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Find Invoke",
                path=f"invokes/find/{name}/not_found",
                description=f"Invoke with {name} `{message.content}` not found",
            )
            return await self.bot_message.edit(embed=embed)

        paginator = ButtonPaginator(
            title="All invokes",
            segments=output,
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction, deferred=True)

    @button(label="TIMESTAMP", style=ButtonStyle.green)
    async def timestamp_button(self, button: Button, interaction: MessageInteraction):
        name = "timestamp"


class Invokes(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="All", style=ButtonStyle.green)
    async def all_invokes_button(self, button: Button, interaction: MessageInteraction):
        paginator = ButtonPaginator(
            title="All invokes",
            segments=pretty(self.bot._commands_ran),
            color=EMBED_COLOR,
            prefix="```yaml\n",
            suffix="```",
            button_style=ButtonStyle.green,
        )
        await paginator.start(interaction)

    @button(label="Find", style=ButtonStyle.green)
    async def find_invoke_button(self, button: Button, interaction: MessageInteraction):
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Find Invoke",
            path=f"invokes/find",
        )

        await interaction.response.defer()
        await self.bot_message.edit(embed=embed, view=FindInvoke(self.ctx))



import pathlib
import os

from . import __version__
from dotenv import load_dotenv

# emojis
THUMBS_UP = "👍"
THUMBS_DOWN = "👎"
ERROR = "❌"

# info

ROOT = pathlib.Path(__file__).parent
VERSION = __version__

BLACKLIST_HELP = """
Send the name of the commands you want to blacklist the user from.
    => Seperate the command names by a comma
    => Send 'all' to blacklist the user from every command
"""
DESCRIPTION = f"""
Welcome to disnake-debug v{VERSION}
-----------------------------
waiting:
    - when the bot sends a question (non-embed message) it is waiting for you to send a response to that question
        => to stop this just type q
"""
PATH_HELP = """
Type the path you want to go to (eg /blacklist/user)
Not all paths are available because some require additional info (eg which guild to leave in leave_guild/leave)
"""
PERMISSION_LIST = """

Everything: 1099511627775
Normal user: 274878221376
Admin: 8
None: 0
"""


class InvalidEnv(Exception):
    """ """


try:
    load_dotenv('./.env')
    os.environ["EMBED_COLOR"]
except KeyError:
    with open("./.env", "a") as env:
        env.write("\n#disnake-debug\nEMBED_COLOR = 0x0000ff")
try:
    load_dotenv('./.env')
    EMBED_COLOR = int(os.environ["EMBED_COLOR"], 16)
except (TypeError, ValueError):
    raise InvalidEnv("EMBED_COLOR must be hex")



from distutils.core import setup
import pathlib  

ROOT = pathlib.Path(__file__).parent
with open(ROOT / "README.md", "r", encoding="utf-8") as f:
    README = f.read()
__version__ = __import__('disnake-debug.__init__').__version__

setup(
    name="disnake-debug",
    packages=["disnake-debug", "disnake-debug.dialogs"],
    version=__version__,
    license="MIT",
    description="A visual debugging extension for disnake discord bots including easy blacklisting, evaluation, echoing and much more!",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Caeden PH",
    author_email="caedenperelliharris@gmail.com",
    url="https://github.com/CaedenPH/disnake-debug",
    download_url=f'https://github.com/CaedenPH/disnake-debug/archive/refs/tags/v_{__version__.replace(".", "")}.tar.gz',
    keywords=["disnake", "discord", "debugging", "visual", "extension", "cog"],
    project_urls={
        "Code": "https://github.com/CaedenPH/disnake-debug",
    },
    install_requires=["disnake", "aiosqlite", "fuzzywuzzy", "python-Levenshtein", "python-dotenv", "disnake-paginator"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from typing import Union
from ..utils import FindSnowflake, EmbedFactory, MainMenu, get_bot_message
from ..constants import THUMBS_UP, ERROR, BLACKLIST_HELP

from disnake import (
    MessageInteraction,
    ButtonStyle,
    User,
    Guild,
    TextChannel,
    Message,
)
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot


class BlacklistSnowflake(View):
    def __init__(
        self,
        ctx: Context,
        snowflake: Union[User, Guild, TextChannel],
        _type: str = "user",
    ):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.snowflake = snowflake
        self._type = _type
        self.add_item(MainMenu(ctx))

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="Blacklist", style=ButtonStyle.green)
    async def blacklist_button(self, button: Button, interaction: MessageInteraction):
        cursor = await self.bot._db.cursor()
        await cursor.execute(
            f"SELECT * FROM blacklist WHERE {self._type}_id = ?", (self.snowflake.id,)
        )
        result = await cursor.fetchall()

        if not result:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Blacklist/Unblacklist",
                path=f"blacklist/{self._type}/choose/commands",
                description=BLACKLIST_HELP,
            )
            await self.bot_message.edit(embed=embed)
            await interaction.response.send_message(
                "What commands do you want to blacklist the user from?", ephemeral=True
            )
            message: Message = await self.bot.wait_for("message", check=self.check)
            if message.content == "q":
                return await interaction.message.reply(
                    "Cancelled the current `wait_for`"
                )

            await message.add_reaction(THUMBS_UP)

            content = ",".join([e.strip() for e in message.content.split(",")])
            if self._type == "user":
                result = (0, 0, self.snowflake.id, content)
            elif self._type == "channel":
                result = (0, self.snowflake.id, 0, content)
            elif self._type == "guild":
                result = (self.snowflake.id, 0, 0, content)

            await cursor.execute("Insert into blacklist values(?, ?, ?, ?)", result)
            await self.bot._db.commit()

            embed = EmbedFactory.static_embed(
                self.ctx,
                "Blacklist/Unblacklist",
                path=f"blacklist/",
                description=f"Blacklisted {self.snowflake.name} from commands: {content}",
            )
            return await self.bot_message.edit(embed=embed, view=Blacklist(self.ctx))

    @button(label="Unblacklist", style=ButtonStyle.green)
    async def unblacklist_button(self, button: Button, interaction: MessageInteraction):
        cursor = await self.bot._db.cursor()
        await cursor.execute(
            f"SELECT * FROM blacklist WHERE {self._type}_id = ?", (self.snowflake.id,)
        )
        result = await cursor.fetchall()

        if not result:
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Blacklist/Unblacklist",
                path=f"unblacklist/{self._type}/notfound",
                description=f"{self.snowflake.name} is not blacklisted from any commands!",
            )
            await self.bot_message.edit(embed=embed)
            return await interaction.response.defer()
        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"unblacklist/{self._type}/",
            description=f"I unblacklisted {self.snowflake.name} from all commands",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.defer()

        cursor = await self.bot._db.cursor()
        await cursor.execute(
            f"DELETE FROM blacklist WHERE {self._type}_id = ?", (self.snowflake.id,)
        )
        await self.bot._db.commit()

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"unblacklist/{self._type}/",
            description=f"I unblacklisted {self.snowflake.name} from all commands",
        )
        await self.bot_message.edit(embed=embed)


class Blacklist(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    @button(label="User", style=ButtonStyle.green)
    async def blacklist_user_button(
        self, button: Button, interaction: MessageInteraction
    ):
        name = "user"

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"blacklist/{name}",
            description="Send an id/name of the user you want to blacklist/unblacklist",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_user(self.bot, message.content)
        if not response:
            other_response = await FindSnowflake.find_any(self.bot, message.content)
            if not other_response:
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Blacklist/Unblacklist",
                    path=f"blacklist/{name}/notfound",
                    description=f"I could not find {name} with id/name {message.content}",
                )
                await message.add_reaction(ERROR)
                return await self.bot_message.edit(embed=embed)
            if not isinstance(other_response, User):
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Blacklist/Unblacklist",
                    path="blacklist/{name}/other",
                    description=f"I could not find a {name} user with id/name {message.content}, but i found a {type(other_response)} {other_response}",
                )
                await message.add_reaction(ERROR)
                return await self.bot_message.edit(embed=embed)

        if isinstance(response, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Blacklist/Unblacklist",
                path=f"blacklist/{name}/overload",
                description=f"You were too broad! I found multiple **{name}s** with name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"blacklist/{name}/choose",
            description=f"I found {response}",
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(
            embed=embed, view=BlacklistSnowflake(self.ctx, response)
        )

    @button(label="Channel", style=ButtonStyle.green)
    async def blacklist_channel_button(
        self, button: Button, interaction: MessageInteraction
    ):
        name = "channel"

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"blacklist/{name}",
            description=f"Send an id/name of the {name} you want to blacklist/unblacklist",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_channel(self.bot, message.content)
        if not response:
            other_response = await FindSnowflake.find_any(self.bot, message.content)
            if not other_response:
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Blacklist/Unblacklist",
                    path=f"blacklist/{name}/notfound",
                    description=f"I could not find {name} with id/name {message.content}",
                )
                await message.add_reaction(ERROR)
                return await self.bot_message.edit(embed=embed)
            if not isinstance(other_response, TextChannel):
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Blacklist/Unblacklist",
                    path="blacklist/{name}/other",
                    description=f"I could not find a {name} with id/name {message.content}, but i found a {type(other_response)} {other_response}",
                )
                await message.add_reaction(ERROR)
                return await self.bot_message.edit(embed=embed)

        if isinstance(response, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Blacklist/Unblacklist",
                path=f"blacklist/{name}/overload",
                description=f"You were too broad! I found multiple **{name}s** with name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"blacklist/{name}/choose",
            description=f"I found {response}",
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(
            embed=embed, view=BlacklistSnowflake(self.ctx, response)
        )

    @button(label="Guild", style=ButtonStyle.green)
    async def blacklist_guild_button(
        self, button: Button, interaction: MessageInteraction
    ):
        name = "guild"

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"blacklist/{name}",
            description=f"Send an id/name of the {name} you want to blacklist/unblacklist",
        )
        await self.bot_message.edit(embed=embed)
        await interaction.response.send_message("What id/name?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        if message.content == "q":
            return await interaction.message.reply("Cancelled the current `wait_for`")

        response = await FindSnowflake.find_guild(self.bot, message.content)
        if not response:
            other_response = await FindSnowflake.find_any(self.bot, message.content)
            if not other_response:
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Blacklist/Unblacklist",
                    path=f"blacklist/{name}/notfound",
                    description=f"I could not find {name} with id/name {message.content}",
                )
                await message.add_reaction(ERROR)
                return await self.bot_message.edit(embed=embed)
            if not isinstance(other_response, Guild):
                embed = EmbedFactory.static_embed(
                    self.ctx,
                    "Blacklist/Unblacklist",
                    path="blacklist/{name}/other",
                    description=f"I could not find a {name} with id/name {message.content}, but i found a {type(other_response)} {other_response}",
                )
                await message.add_reaction(ERROR)
                return await self.bot_message.edit(embed=embed)

        if isinstance(response, list):
            embed = EmbedFactory.static_embed(
                self.ctx,
                "Blacklist/Unblacklist",
                path=f"blacklist/{name}/overload",
                description=f"You were too broad! I found multiple **{name}s** with name {message.content}",
            )
            await message.add_reaction(ERROR)
            return await self.bot_message.edit(embed=embed)

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Blacklist/Unblacklist",
            path=f"blacklist/{name}/choose",
            description=f"I found {response}",
        )
        await message.add_reaction(THUMBS_UP)
        await self.bot_message.edit(
            embed=embed, view=BlacklistSnowflake(self.ctx, response)
        )



import os
import asyncio

from ..utils import EmbedFactory, MainMenu, get_bot_message
from disnake import Message, MessageInteraction, ButtonStyle
from disnake.ui import View, Button, button
from disnake.ext.commands import Context, Bot


class Github(View):
    def __init__(self, ctx: Context):
        super().__init__()

        self.ctx = ctx
        self.bot: Bot = ctx.bot
        self.path = os.getcwd()
        self.bot_message = get_bot_message(ctx)
        self.add_item(MainMenu(ctx))

    async def interaction_check(self, interaction: MessageInteraction) -> bool:
        return (
            interaction.author == self.ctx.author
            and interaction.channel == self.ctx.channel
        )

    def check(self, m: Message) -> bool:
        return m.author == self.ctx.author and m.channel == self.ctx.channel

    @button(label="Push", style=ButtonStyle.green)
    async def push_button(self, button: Button, interaction: MessageInteraction):
        if not ".git" in os.listdir(self.path):
            return await interaction.response.send_message(
                "This command will not working because this is not a github directory (missing .git)"
            )

        embed = EmbedFactory.static_embed(
            self.ctx,
            "Git push",
            path="github/push",
            description="Pushing to github...What is the reason for this commit?",
        )
        await interaction.response.send_message("What reason?", ephemeral=True)

        message: Message = await self.bot.wait_for("message", check=self.check)
        git_commands = [
            ["git", "add", "."],
            ["git", "commit", "-m", message.content],
            ["git", "push"],
        ]

        for git_command in git_commands:
            process = await asyncio.create_subprocess_exec(
                git_command[0],
                *git_command[1:],
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )

            output, error = await process.communicate()
            embed.description += f'[{" ".join(git_command)!r} exited with return code ```{process.returncode}```\n'

            if output:
                embed.description += f"**[stdout]**\n```yaml\n{output.decode()}```\n"
            if error:
                embed.description += f"**[stderr]**\n```yaml\n{error.decode()}```\n"
        await self.bot_message.edit(embed=embed)

    @button(label="Pull", style=ButtonStyle.green)
    async def pull_button(self, button: Button, interaction: MessageInteraction):
        if not ".git" in os.listdir(self.path):
            return await interaction.response.send_message(
                "This command will not working because this is not a github directory (missing .git)"
            )

        await interaction.response.defer()

        embed = EmbedFactory.static_embed(
            self.ctx, "Git pull", path="github/pull", description="Pulling from github"
        )
        git_commands = [["git", "stash"], ["git", "pull", "--ff-only"]]

        for git_command in git_commands:
            process = await asyncio.create_subprocess_exec(
                git_command[0],
                *git_command[1:],
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )

            output, error = await process.communicate()
            embed.description += f'[{" ".join(git_command)!r} exited with return code {process.returncode}\n'

            if output:
                embed.description += f"**[stdout]**\n```yaml\n{output.decode()}```\n"
            if error:
                embed.description += f"**[stderr]**\n```yaml\n{error.decode()}```\n"
        await self.bot_message.edit(embed=embed)
